@@ -1,19 +1,16 @@
-
-/*  $Header$
- *
- *  Copyright (C) 2004-2008 Sami Pietila
+/*  Copyright (c) 2004-2008 Sami Pietila
  *  Copyright (c) 2008-2009 Robert Ancell
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *           
- *  This program is distributed in the hope that it will be useful, but 
+ *
+ *  This program is distributed in the hope that it will be useful, but
  *  WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  *  General Public License for more details.
- *           
+ *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
@@ -28,13 +25,26 @@
 
 extern int _mp_equation_parse(yyscan_t yyscanner);
 
+
+char *
+utf8_next_char (const char *c)
+{
+    c++;
+    while ((*c & 0xC0) == 0x80)
+        c++;
+    return (char *)c;
+}
+
+
 static int
 get_variable(MPEquationParserState *state, const char *name, MPNumber *z)
 {
     int result = 1;
-    
+
     if (strcmp(name, "e") == 0)
         mp_get_eulers(z);
+    else if (strcmp(name, "i") == 0)
+        mp_get_i(z);
     else if (strcmp(name, "π") == 0)
         mp_get_pi(z);
     else if (state->options->get_variable)
@@ -42,15 +52,44 @@ get_variable(MPEquationParserState *state, const char *name, MPNumber *z)
     else
         result = 0;
 
+    /* If has more than one character then assuming a multiplication of variables */
+    if (!result && utf8_next_char(name)[0] != '\0') {
+        const char *c, *next;
+        char *buffer = malloc(sizeof(char) * strlen(name));
+        MPNumber value;
+
+        result = 1;
+        mp_set_from_integer(1, &value);
+        for (c = name; *c != '\0'; c = next)
+        {
+            MPNumber t;
+
+            next = utf8_next_char(c);
+            snprintf(buffer, next - c + 1, "%s", c);
+
+            if (!get_variable(state, buffer, &t))
+            {
+                result = 0;
+                break;
+            }
+            mp_multiply(&value, &t, &value);
+        }
+
+        free(buffer);
+        if (result)
+            mp_set_from_mp(&value, z);
+    }
+
     return result;
 }
 
 static void
 set_variable(MPEquationParserState *state, const char *name, const MPNumber *x)
 {
+    // Reserved words, e, π, mod, and, or, xor, not, abs, log, ln, sqrt, int, frac, sin, cos, ...
     if (strcmp(name, "e") == 0 || strcmp(name, "π") == 0)
         return; // FALSE
-    
+
     if (state->options->set_variable)
         state->options->set_variable(name, x, state->options->callback_data);
 }
@@ -61,15 +100,16 @@ set_variable(MPEquationParserState *state, const char *name, const MPNumber *x)
 // letters+numbers = numbers+letters+numbers = function
 
 
-static int sub_atoi(const char *data)
+int
+sub_atoi(const char *data)
 {
     int i, value = 0;
     const char *digits[] = {"₀", "₁", "₂", "₃", "₄", "₅", "₆", "₇", "₈", "₉", NULL};
-    
+
     do {
         for(i = 0; digits[i] != NULL && strncmp(data, digits[i], strlen(digits[i])) != 0; i++);
         if(digits[i] == NULL)
-            return -1;
+            return 0;
         data += strlen(digits[i]);
         value = value * 10 + i;
     } while(*data != '\0');
@@ -77,24 +117,40 @@ static int sub_atoi(const char *data)
     return value;
 }
 
+int
+super_atoi(const char *data)
+{
+   int i, value = 0;
+   const char *digits[11] = {"⁰", "¹", "²", "³", "⁴", "⁵", "⁶", "⁷", "⁸", "⁹", NULL};
+
+   do {
+      for(i = 0; digits[i] != NULL && strncmp(data, digits[i], strlen(digits[i])) != 0; i++);
+      if(digits[i] == NULL)
+         return 0;
+      value = value * 10 + i;
+      data += strlen(digits[i]);
+   } while(*data != '\0');
+
+   return value;
+}
 
 static int
 get_function(MPEquationParserState *state, const char *name, const MPNumber *x, MPNumber *z)
 {
     char *c, *lower_name;
     int result = 1;
-    
+
     lower_name = strdup(name);
     for (c = lower_name; *c; c++)
         *c = tolower(*c);
-    
+
     // FIXME: Re Im ?
 
     if (strcmp(lower_name, "log") == 0)
         mp_logarithm(10, x, z); // FIXME: Default to ln
     else if (strncmp(lower_name, "log", 3) == 0) {
         int base;
-        
+
         base = sub_atoi(lower_name + 3);
         if (base < 0)
             result = 0;
@@ -116,13 +172,13 @@ get_function(MPEquationParserState *state, const char *name, const MPNumber *x,
     else if (strcmp(lower_name, "cos") == 0)
         mp_cos(x, state->options->angle_units, z);
     else if (strcmp(lower_name, "tan") == 0)
-        mp_tan(x, state->options->angle_units, z);    
+        mp_tan(x, state->options->angle_units, z);
     else if (strcmp(lower_name, "sin⁻¹") == 0 || strcmp(lower_name, "asin") == 0)
         mp_asin(x, state->options->angle_units, z);
     else if (strcmp(lower_name, "cos⁻¹") == 0 || strcmp(lower_name, "acos") == 0)
         mp_acos(x, state->options->angle_units, z);
     else if (strcmp(lower_name, "tan⁻¹") == 0 || strcmp(lower_name, "atan") == 0)
-        mp_atan(x, state->options->angle_units, z);    
+        mp_atan(x, state->options->angle_units, z);
     else if (strcmp(lower_name, "sinh") == 0)
         mp_sinh(x, z);
     else if (strcmp(lower_name, "cosh") == 0)
@@ -143,15 +199,15 @@ get_function(MPEquationParserState *state, const char *name, const MPNumber *x,
         result = state->options->get_function(name, x, z, state->options->callback_data);
     else
         result = 0;
-    
+
     free(lower_name);
-    
+
     return result;
 }
 
 
-int
-mp_equation_parse(const char *expression, MPEquationOptions *options, MPNumber *result)
+MPErrorCode
+mp_equation_parse(const char *expression, MPEquationOptions *options, MPNumber *result, char **error_token)
 {
     int ret;
     MPEquationParserState state;
@@ -159,7 +215,7 @@ mp_equation_parse(const char *expression, MPEquationOptions *options, MPNumber *
     YY_BUFFER_STATE buffer;
 
     if (!(expression && result) || strlen(expression) == 0)
-        return -EINVAL;
+        return PARSER_ERR_INVALID;
 
     memset(&state, 0, sizeof(MPEquationParserState));
     state.options = options;
@@ -174,24 +230,52 @@ mp_equation_parse(const char *expression, MPEquationOptions *options, MPNumber *
     buffer = _mp_equation__scan_string(expression, yyscanner);
 
     ret = _mp_equation_parse(yyscanner);
+    if (state.error_token != NULL && error_token != NULL) {
+        *error_token = state.error_token;
+    }
 
     _mp_equation__delete_buffer(buffer, yyscanner);
     _mp_equation_lex_destroy(yyscanner);
 
     /* Failed to parse */
     if (ret)
-        return -PARSER_ERR_INVALID;
-        
+        return PARSER_ERR_INVALID;
+
     /* Error during parsing */
     if (state.error)
         return state.error;
 
     if (mp_get_error())
-        return -PARSER_ERR_MP;
+        return PARSER_ERR_MP;
 
     mp_set_from_mp(&state.ret, result);
 
-    return 0;
+    return PARSER_ERR_NONE;
+}
+
+
+const char *
+mp_error_code_to_string(MPErrorCode error_code)
+{
+    switch(error_code)
+    {
+    case PARSER_ERR_NONE:
+        return "PARSER_ERR_NONE";
+    case PARSER_ERR_INVALID:
+        return "PARSER_ERR_INVALID";
+    case PARSER_ERR_OVERFLOW:
+        return "PARSER_ERR_OVERFLOW";
+    case PARSER_ERR_UNKNOWN_VARIABLE:
+        return "PARSER_ERR_UNKNOWN_VARIABLE";
+    case PARSER_ERR_UNKNOWN_FUNCTION:
+        return "PARSER_ERR_UNKNOWN_FUNCTION";
+    case PARSER_ERR_UNKNOWN_CONVERSION:
+        return "PARSER_ERR_UNKNOWN_CONVERSION";
+    case PARSER_ERR_MP:
+        return "PARSER_ERR_MP";
+    default:
+        return "Unknown parser error";
+    }
 }
 
 
